/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase.actions;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.geom.Line2D;
import java.util.*;

import javax.swing.*;

import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.fsa.FSAContainer;
import de.uni_paderborn.fujaba.fsa.FSAObject;
import de.uni_paderborn.fujaba.fsa.swing.JLine;
import de.uni_paderborn.fujaba.uml.*;
import de.uni_paderborn.fujaba.usecase.*;


/**
 * Place all Actors outside the System box.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.11 $
 */
public class PlaceActorsOutsideSystemAction extends AbstractAction
{
   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private int MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR = 20;


   /**
    * Sets the minimumSpaceBetweenSystemAndActor attribute of the PlaceActorsOutsideSystemAction
    * object
    *
    * @param space  The new minimumSpaceBetweenSystemAndActor value
    */
   public void setMinimumSpaceBetweenSystemAndActor (int space)
   {
      if (space > 0)
      {
         MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR = space;
      }
      else
      {
         throw new IllegalArgumentException
             (space + " pixels is not enough space between System Box and Actors");
      }
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param event  No description provided
    */
   public void actionPerformed (ActionEvent event)
   {
      UMLDiagram diagram = UMLProject.get().getCurrentUMLDiagram();

      // find out position and size of system box, collect actors
      Rectangle systemBounds = new Rectangle();
      Vector actors = new Vector();

      Iterator iter = diagram.iteratorOfElements();
      while (iter.hasNext())
      {
         ASGElement asgElement = (ASGElement) iter.next();

         if (asgElement instanceof UsecaseSystem)
         {
            FSAContainer fsa = Utility.getMainFSAContainer (asgElement);
            systemBounds = new Rectangle (fsa.getJComponent().getBounds());
         }
         if (asgElement instanceof UsecaseActor)
         {
            actors.add (asgElement);
         }
      }

      for (int i = 0; i < actors.size(); ++i)
      {
         UsecaseActor actor = (UsecaseActor) actors.get (i);
         FSAContainer actorFSA = Utility.getMainFSAContainer (actor);
         JComponent actorJComponent = actorFSA.getJComponent();
         Rectangle actorBounds = actorJComponent.getBounds();

         // only move the Actor somewhere else if he is (partially)
         // below the System box
         if (actorBounds.intersects (systemBounds))
         {
            // Compute where to move the Actor.  She is moved next to her
            // closest side of the System box.

            int actorLeft = actorBounds.x;
            int actorRight = actorBounds.x + actorBounds.width;
            int actorTop = actorBounds.y;
            int actorBottom = actorBounds.y + actorBounds.height;

            int systemLeft = systemBounds.x;
            int systemRight = systemBounds.x + systemBounds.width;
            int systemTop = systemBounds.y;
            int systemBottom = systemBounds.y + systemBounds.height;

            int leftDistance = Math.abs (actorRight - systemLeft);
            int topDistance = Math.abs (actorBottom - systemTop);
            int rightDistance = Math.abs (actorLeft - systemRight);
            int bottomDistance = Math.abs (actorTop - systemBottom);

            int[] distances =
               {leftDistance, topDistance,
               rightDistance, bottomDistance};
            Arrays.sort (distances);
            int minDistance = distances[0];

            final int TOP = 0;
            final int RIGHT = 1;
            final int BOTTOM = 2;
            final int LEFT = 3;
            int direction = LEFT;

            // if any two distances are the same, the order of the
            // if-statements determines the placement: left, top, right,
            // bottom
            if (minDistance == leftDistance)
            {
               direction = LEFT;
            }
            else if (minDistance == topDistance)
            {
               direction = TOP;
            }
            else if (minDistance == rightDistance)
            {
               direction = RIGHT;
            }
            else if (minDistance == bottomDistance)
            {
               direction = BOTTOM;
            }
            else
            {
               /*
                *  throw an exception?
                */
            }

            Point desiredNewLoc = new Point (actorJComponent.getLocation());
            switch (direction)
            {
               case LEFT:
                  desiredNewLoc.translate (-leftDistance, 0);
                  desiredNewLoc.translate (-MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR, 0);
                  break;
               case TOP:
                  desiredNewLoc.translate (0, -topDistance);
                  desiredNewLoc.translate (0, -MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR);
                  break;
               case RIGHT:
                  desiredNewLoc.translate (rightDistance, 0);
                  desiredNewLoc.translate (MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR, 0);
                  break;
               case BOTTOM:
                  desiredNewLoc.translate (0, bottomDistance);
                  desiredNewLoc.translate (0, MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR);
                  break;
               default:
                  // throw an exception?
                  break;
            }

            // Now check for space at desiredNewLoc (other actors or
            // usages may be there).

            // Strategy: a rectangle actorShape as large as an Actor will
            // be placed at every location tryLoc on the rectangle
            // tryRect's edge.  This rectangle's edge will be just around
            // the system box considering minimum space between actor and
            // system box.  At every location tryLoc it is checked if
            // actorShape overlaps with any other actor or usage.  If it
            // doesn't, the actor is placed there, otherwise the next
            // location is checked.  tryLoc starts at desirednewLoc and
            // advances clockwise around tryRect.  If there isn't any
            // location on tryRect's edge with enough space for the
            // actor, tryRect is enlarged by the actor's size, every
            // location on this new tryRect will be checked and so on.
            // The checking of every location is actually done with two
            // tryLocs tryLoc1 and tryLoc2.  First, the location to the
            // closest sytem box side is calculated.  Then tryLoc1 walks
            // around clockwise and tryLoc2 walks around counterclockwise
            // the system box and the two are checked after every
            // advance.  This usually places the actor closer to where
            // you would expect it.


            // put a Rectangle as large as an Actor to the desired location
            Rectangle actorShape = new Rectangle (actorBounds);
            actorShape.setLocation (desiredNewLoc);

            Rectangle tryRect = new Rectangle (systemBounds);

            // on the left and upper side tryRect needs to be enlarged by
            // the actors width and height respectively plus space
            // between actor and system box
            int actorWidth = actorBounds.width;
            int actorHeight = actorBounds.height;
            Point addUpperLeftPoint = new Point (tryRect.getLocation());
            addUpperLeftPoint.translate (-actorWidth, -actorHeight);
            addUpperLeftPoint.translate (-MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR,
               -MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR);
            tryRect.add (addUpperLeftPoint);

            // same on right and bottom
            Point addLowerRightPoint = new Point (systemRight + 1,
               systemBottom + 1);
            addLowerRightPoint.translate (MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR,
               MIN_SPACE_BETWEEN_SYSTEM_AND_ACTOR);
            tryRect.add (addLowerRightPoint);

            // now the actor will be placed at every position on
            // tryRect's edge checking if there is enough space

            Point tryLoc1 = new Point (desiredNewLoc);
            Point tryLoc2 = new Point (desiredNewLoc);
            boolean foundLoc = false;
            do
            {
               // try every position
               boolean allLocsChecked = false;
               do
               {
                  actorShape.setLocation (tryLoc1);
                  if (rectangleIntersectsOtherActorsOrUsages
                      (actorShape, diagram.iteratorOfElements(), actor))
                  {
                     advanceClockwise (tryLoc1, tryRect);
                     if (tryLoc1.equals (tryLoc2))
                     {
                        allLocsChecked = true;
                     }

                     actorShape.setLocation (tryLoc2);
                     if (rectangleIntersectsOtherActorsOrUsages
                         (actorShape, diagram.iteratorOfElements(), actor))
                     {
                        advanceCounterClockwise (tryLoc2, tryRect);
                        if (tryLoc2.equals (tryLoc1))
                        {
                           allLocsChecked = true;
                        }
                     }
                     else
                     {
                        foundLoc = true;
                        actorFSA.setLocation (tryLoc2);
                     }
                  }
                  else
                  {
                     foundLoc = true;
                     actorFSA.setLocation (tryLoc1);
                  }
               } while (!allLocsChecked && !foundLoc);

               // try more distance if !foundLoc
               if (!foundLoc)
               {
                  int addSpace = 10;

                  addUpperLeftPoint = new Point (tryRect.getLocation());
                  addUpperLeftPoint.translate (-actorWidth - addSpace,
                     -actorHeight - addSpace);
                  tryRect.add (addUpperLeftPoint);

                  addLowerRightPoint = new Point (tryRect.getLocation());
                  addLowerRightPoint.translate (tryRect.width, tryRect.height);
                  addLowerRightPoint.translate (actorWidth + addSpace,
                     actorHeight + addSpace);
                  tryRect.add (addLowerRightPoint);

                  switch (direction)
                  {
                     case LEFT:
                        desiredNewLoc.translate (-actorWidth - addSpace, 0);
                        break;
                     case TOP:
                        desiredNewLoc.translate (0, -actorHeight - addSpace);
                        break;
                     case RIGHT:
                        desiredNewLoc.translate (actorWidth + addSpace, 0);
                        break;
                     case BOTTOM:
                        desiredNewLoc.translate (0, actorHeight + addSpace);
                        break;
                     default:
                        // throw an exception?
                        break;
                  }
                  tryLoc1 = new Point (desiredNewLoc);
                  tryLoc2 = new Point (desiredNewLoc);
               }
            } while (!foundLoc);
         } // if actor below system box

      } // for all actors

      UMLProject.get().refreshDisplay();
   } // end of newActionPerformed( ... )


   /**
    * Check for intersection with any other Actor or Usage except the item except.
    *
    * @param r       No description provided
    * @param items   No description provided
    * @param except  No description provided
    * @return        No description provided
    */
   private static boolean rectangleIntersectsOtherActorsOrUsages (Rectangle r,
                                                                  Iterator items,
                                                                  UMLDiagramItem except)
   {
      boolean intersects = false;
      while (items.hasNext() && !intersects)
      {
         ASGElement asgElement = (ASGElement) items.next();

         if (asgElement == except)
         {
            continue;
         }
         if (! (asgElement instanceof UsecaseActor || asgElement instanceof UsecaseUsage))
         {
            continue;
         }

         FSAObject fsa;
         // special check for UsecaseUsages, as lines usually occupy far
         // less space than their bounds
         if (asgElement instanceof UsecaseUsage)
         {
            fsa = Utility.getFSAObject (asgElement, "line");
            JLine jline = (JLine) fsa.getJComponent();
            if (r.intersectsLine (new Line2D.Double (jline.getStartPoint(),
               jline.getEndPoint())))
            {
               intersects = true;
            }
         }
         // for every other item (only UsecaseActors are checked, see
         // above) simply test the bounds for intersection
         else
         {
            fsa = Utility.getMainFSAContainer (asgElement);
            Rectangle itemBounds = fsa.getJComponent().getBounds();
            if (r.intersects (itemBounds))
            {
               intersects = true;
            }
         }
      }
      return intersects;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param p  No description provided
    * @param r  No description provided
    */
   private static void advanceClockwise (Point p, Rectangle r)
   {
      int top = r.y;
      int right = r.x + r.width - 1;
      int bottom = r.y + r.height - 1;
      int left = r.x;

      // move on top side to the right
      if ( (p.y == top) &&  (p.x < right))
      {
         p.translate (1, 0);
      }
      // move on right side to the bottom
      else if ( (p.x == right) &&  (p.y < bottom))
      {
         p.translate (0, 1);
      }
      // move on bottom side to the left
      else if ( (p.y == bottom) &&  (p.x > left))
      {
         p.translate (-1, 0);
      }
      // move on left side to the top
      else if ( (p.x == left) &&  (p.y > top))
      {
         p.translate (0, -1);
      }
      else
      {
         throw new IllegalArgumentException ("Point '" + p
            + "' not on Rectangle '" + r
            + "'s edge!");
      }
      return;
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @param p  No description provided
    * @param r  No description provided
    */
   private static void advanceCounterClockwise (Point p, Rectangle r)
   {
      int top = r.y;
      int right = r.x + r.width - 1;
      int bottom = r.y + r.height - 1;
      int left = r.x;

      // move on top side to the left
      if ( (p.y == top) &&  (p.x > left))
      {
         p.translate (-1, 0);
      }
      // move on left side to the bottom
      else if ( (p.x == left) &&  (p.y < bottom))
      {
         p.translate (0, 1);
      }
      // move on bottom side to the right
      else if ( (p.y == bottom) &&  (p.x < right))
      {
         p.translate (1, 0);
      }
      // move on right side to the top
      else if ( (p.x == right) &&  (p.y > top))
      {
         p.translate (0, -1);
      }
      else
      {
         throw new IllegalArgumentException ("Point '" + p
            + "' not on Rectangle '" + r
            + "'s edge!");
      }
      return;
   }

}

/*
 * $Log: PlaceActorsOutsideSystemAction.java,v $
 * Revision 1.11  2004/10/20 17:50:29  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
